@@ -9,7 +9,7 @@ disk can be accessed through the **Array** constructor. To remove a Darr
 array from disk, use **delete_array**.
 
 """
-
+# TODO replace distutils (is deprecated) with packaging
 import distutils.version
 import json
 import os
@@ -22,7 +22,7 @@ from pathlib import Path
 from .datadir import DataDir, create_datadir
 from .metadata import MetaData
 from .numtype import arrayinfotodtype, arraynumtypeinfo, numtypesdescr
-from .readcodearray import readcode, readcodefunc
+from .readcodearray import readcode, readcodefunc, shapeexplanationtextarray
 from .utils import fit_frames, wrap, check_accessmode, product, tempdirfile
 from ._version import get_versions
 
@@ -156,6 +156,17 @@ class Array:
         """Total number of values in the data array."""
         return int(self._size)
 
+    @property
+    def readcodelanguages(self):
+        """Tuple of the languages that the `readcode` method can produce
+        reading code for. Code in these languages is also included in the
+        README.txt file that is stored as part of the array ."""
+        languages = []
+        for lang in readcodefunc.keys():
+            if readcode(self, language=lang) is not None:
+                languages.append(lang)
+        return tuple(sorted(languages))
+
     def __getitem__(self, index):
         with self._open_array() as (ar, _):
             values = np.array(ar[index], copy=True)
@@ -639,7 +650,7 @@ class Array:
                        accessmode=accessmode, metadata=metadata,
                        chunklen=chunklen, overwrite=overwrite)
 
-    def readcode(self, language):
+    def readcode(self, language, abspath=False, basepath=None):
         """Generate code to read the array in a different language.
 
         Note that this does not include reading the metadata, which is just
@@ -651,6 +662,13 @@ class Array:
             One of the languages that are supported. Choose from:
             'darr', 'idl', 'julia_ver0', 'julia_ver1', 'mathematica',
             'matlab', 'maple', 'numpy', 'numpymemmap', 'R'.
+        abspath: bool
+            Should the paths to the data files be absolute or not? Default:
+            True.
+        basepath: str or pathlib.Path or None
+            Path relative to which the binary array data file should be
+            provided. Default: None.
+
         Example
         -------
         >>> import darr
@@ -664,10 +682,40 @@ class Array:
         if language not in readcodefunc.keys():
             raise ValueError(f'Language "{language}" not supported, choose '
                              f'from {readcodefunc.keys()}')
-        d = self._arrayinfo
-        return readcodefunc[language](numtype=d['numtype'],
-                                      shape=d['shape'],
-                                      endianness=d['byteorder'])
+        return readcode(self, language=language, basepath=basepath,
+                        abspath=abspath)
+
+    def archive(self, filepath=None, compressiontype='xz', overwrite=False):
+        """Archive array data into a single compressed file.
+
+        Parameters
+        ----------
+        filepath: str
+            Name of the archive. In None, it will be derived from the data's
+            path name.
+        compressiontype: str
+            One of 'xz', 'gz', or 'bz2', corresponding to the gzip, bz2 and
+            lzma compression algorithms supported by the Python standard
+            library.
+        overwrite: (True, False), optional
+            Overwrites existing archive if it exists. Default is False.
+
+        Returns
+        -------
+        pathlib.Path
+            The path of the created archive
+
+        Notes
+        -----
+        See the `tarfile library`_ for more info on archiving formats
+
+        .. _tarfile library:
+           https://docs.python.org/3/library/tarfile.html
+
+        """
+        return self._datadir.archive(filepath=filepath,
+                                     compressiontype=compressiontype,
+                                     overwrite=overwrite)
 
 
 def _fillgenerator(shape, dtype='float64', fill=0., fillfunc=None,
@@ -1141,9 +1189,17 @@ def readcodetxt(da):
     """
 
     s = numtypedescriptiontxt(da)
-    s += "Example code for reading the numeric data\n" \
-         "=========================================\n\n"
+    s += "Code snippets for reading the numeric data\n" \
+         "==========================================\n\n"
+    if len(da.shape) > 1:
+        s += wrap(f"Note that the array is multi-dimensional, and stored "
+                  f"with a row-major memory layout. In column-major "
+                  f"languages (see Note below), the code provided here will "
+                  f"lead to an array that has its dimensions inversed "
+                  f"{da.shape[::-1]} with respect to the format description "
+                  f"above {da.shape}.") + '\n\n'
     languages = (
+        ("Python:", 'python'),
         ("Python with Darr:", "darr"),
         ("Python with Numpy:", "numpy"),
         ("Python with Numpy (memmap):", "numpymemmap"),
@@ -1159,7 +1215,11 @@ def readcodetxt(da):
         codetext = readcode(da, language)
         if codetext is not None:
             s += f"{heading}\n{'-'*len(heading)}\n{codetext}\n"
-    return s
+    if len(da.shape) > 1:
+        return f'{s}\n{shapeexplanationtextarray}'
+    else:
+        return f'{s}\n'
+
 
 
 def numtypedescriptiontxt(da):
@@ -1174,6 +1234,10 @@ def numtypedescriptiontxt(da):
     """
     d = da._arrayinfo
     shape = d['shape']
+    if len(shape) > 1:
+        ismultid = True
+    else:
+        ismultid = False
     typedescr = numtypesdescr[d['numtype']]
     arrayorder = d['arrayorder']
     endianness = d['byteorder']
@@ -1190,12 +1254,13 @@ def numtypedescriptiontxt(da):
     else:
         raise ValueError(f'arrayorder type "{arrayorder}" unknown')
     s = wrap("This directory contains a numeric array that is stored in an "
-             "open and simple format and that is easy to access in most "\
-             "analysis environments. In Python, you can use the Darr "\
-             "library (https://pypi.org/project/darr/), which was used to "\
-             "create the data. Alternatively, to access the data directly in "
-             "Python or other environments, use the information below, "\
-             "which includes code for a number of popular platforms.")  + "\n\n\n"
+             "open and simple format. It should be easy to access the data in "
+             "most analysis environments. The array can be read using the "
+             "NumPy-based Python library Darr (https://pypi.org/project/darr/), "
+             "which was used to create the data. Alternatively, you can access "
+             "the data directly using the code snippets below. If your "
+             "language is not included, the full data format description "
+             "should help.") + "\n\n"
     s+= f"Data format\n===========\n\n"
     s += wrap("The file 'arrayvalues.bin' contains the raw binary values of "
               "the numeric array, without header information, in the "
@@ -1206,8 +1271,9 @@ def numtypedescriptiontxt(da):
         s += f"  Array length: {shape[0]}\n"
     else:
         s += f"  Array dimensions: {shape}\n"
-    s += f"  Array order layout:  {arrayorder} ({arrayorderdescr})\n\n"
-    s += wrap("These details are also stored in JSON format in the separate "
+    if ismultid:  # we include arrayorder info only if array is multidimensional
+        s += f"  Array order layout:  {arrayorder} ({arrayorderdescr})\n"
+    s += wrap("\nThese details are also stored in JSON format in the separate "
               "UTF-8 text file, 'arraydescription.json'.") + "\n\n"
     if len(da.metadata) > 0:
         s += wrap("The file 'metadata.json' contains metadata in JSON UTF-8 "\
