@@ -4,12 +4,12 @@ from contextlib import contextmanager
 import numpy as np
 from ._version import get_versions
 
-from .array import Array, MetaData, asarray, \
-    check_accessmode, delete_array, create_array, \
-    truncate_array
+from .array import Array, asarray, check_accessmode, delete_array, \
+    create_array, truncate_array
 from .datadir import DataDir, create_datadir
 from .metadata import MetaData
-from .readcoderaggedarray import readcode, readcodefunc
+from .readcoderaggedarray import readcode, readcodefunc, \
+    shapeindexexplanationtextraggedarray
 from .utils import wrap
 
 __all__ = ['RaggedArray', 'asraggedarray', 'create_raggedarray',
@@ -117,15 +117,24 @@ class RaggedArray:
 
     @property
     def size(self):
-        """Total number of values in the data array.
-
-        """
+        """Total number of values in the data array."""
         return int(self._values._size)
 
+    @property
+    def readcodelanguages(self):
+        """Tuple of the languages that the `readcode` method can produce
+                reading code for. Code in these languages is also included in the
+                README.txt file that is stored as part of the array ."""
+        languages = []
+        for lang in readcodefunc.keys():
+            if readcode(self, lang) is not None:
+                languages.append(lang)
+        return tuple(sorted(languages))
+
     def __getitem__(self, item):
         if not np.issubdtype(type(item), np.integer):
             raise TypeError("Only integers can be used for indexing " \
-                            "darraylists, which '{}' is not".format(item))
+                            "RaggedArrays, which '{}' is not".format(item))
         index = slice(*self._indices[item])
         return self._values[index]
 
@@ -133,8 +142,8 @@ class RaggedArray:
         return self._indices.shape[0]
 
     def __repr__(self):
-        return f'darr ragged array ({self.narrays} arrays with at' \
-               f'om shape {self.atom}, {self.accessmode})'
+        return f'RaggedArray ({self.narrays} subarrays with atom shape '\
+               f'{self.atom}, {self.accessmode})'
 
     __str__ = __repr__
 
@@ -205,17 +214,23 @@ class RaggedArray:
         self._update_arraydescr(len=len(self._indices),
                                 size=self._values.size)
 
-    def readcode(self, language):
+    def readcode(self, language, abspath=False, basepath=None):
         """Generate code to read the array in a different language.
 
         Note that this does not include reading the metadata, which is just
         based on a text file in JSON format.
 
-        Parameter
-        ---------
+        Parameters
+        ----------
         language: str
             One of the languages that are supported. Choose from:
             'matlab', 'numpymemmap', 'R'.
+        abspath: bool
+            Should the paths to the data files be absolute or not? Default:
+            True.
+        basepath: str or pathlib.Path or None
+            Path relative to which the binary array data file should be
+            provided. Default: None.
 
         Example
         -------
@@ -237,14 +252,51 @@ class RaggedArray:
         if language not in readcodefunc.keys():
             raise ValueError(f'Language "{language}" not supported, choose '
                              f'from {readcodefunc.keys()}')
-        d = self._arrayinfo
-        return readcodefunc[language](self)
+        return readcode(self, language, basepath=basepath, abspath=abspath)
+
+    def archive(self, filepath=None, compressiontype='xz', overwrite=False):
+        """Archive array data into a single compressed file.
+
+        Parameters
+        ----------
+        filepath: str
+            Name of the archive. In None, it will be derived from the data's
+            path name.
+        compressiontype: str
+            One of 'xz', 'gz', or 'bz2', corresponding to the gzip, bz2 and
+            lzma compression algorithms supported by the Python standard
+            library.
+        overwrite: (True, False), optional
+            Overwrites existing archive if it exists. Default is False.
+
+        Returns
+        -------
+        pathlib.Path
+            The path of the created archive
+
+        Notes
+        -----
+        See the `tarfile library`_ for more info on archiving formats
+
+        .. _tarfile library:
+           https://docs.python.org/3/library/tarfile.html
+
+        """
+        return self._datadir.archive(filepath=filepath,
+                                     compressiontype=compressiontype,
+                                     overwrite=overwrite)
 
 
 # FIXME empty arrayiterable
+# TODO describe int32 vs int64 index
 def asraggedarray(path, arrayiterable, dtype=None, metadata=None,
-                  accessmode='r+', overwrite=False):
+                  accessmode='r+', indextype='int64', overwrite=False):
     path = Path(path)
+    supportedindextypes = ('int8','uint8', 'int16', 'uint16', 'int32',
+                           'uint32', 'int64')
+    if not indextype in supportedindextypes:
+        raise ValueError(f'`indextype` {indextype} not one of '
+                         f'{supportedindextypes}')
     if not hasattr(arrayiterable, 'next'):
         arrayiterable = (a for a in arrayiterable)
     bd = create_datadir(path=path, overwrite=overwrite)
@@ -256,7 +308,7 @@ def asraggedarray(path, arrayiterable, dtype=None, metadata=None,
                        accessmode='r+', overwrite=overwrite)
     firstindices = [[0, len(firstarray)]]
     indicesda = asarray(path=indicespath, array=firstindices,
-                        dtype=np.int64, accessmode='r+',
+                        dtype=indextype, accessmode='r+',
                         overwrite=overwrite)
     valueslen = firstindices[0][1]
     indiceslen = 1
@@ -293,7 +345,7 @@ def asraggedarray(path, arrayiterable, dtype=None, metadata=None,
 
 
 def create_raggedarray(path, atom=(), dtype='float64', metadata=None,
-                       accessmode='r+', overwrite=False):
+                       accessmode='r+', indextype='int64', overwrite=False):
     if not hasattr(atom, '__len__'):
         raise TypeError(f'shape "{atom}" is not a sequence of dimensions.\n'
                         f'If you want just a list of 1-dimensional arrays, '
@@ -301,7 +353,8 @@ def create_raggedarray(path, atom=(), dtype='float64', metadata=None,
     shape = [0] + list(atom)
     ar = np.zeros(shape, dtype=dtype)
     ra = asraggedarray(path=path, arrayiterable=[ar], metadata=metadata,
-                        accessmode=accessmode, overwrite=overwrite)
+                       accessmode=accessmode, indextype=indextype,
+                       overwrite=overwrite)
     # the current ragged array has one element, which is an empty array
     # but we want an empty ragged array => we should get rid of the indices
     create_array(path=ra._indicespath, shape=(0,2), dtype=np.int64,
@@ -311,48 +364,80 @@ def create_raggedarray(path, atom=(), dtype='float64', metadata=None,
 
 # TODO, simplify explanation if subarrays are 1-dimensional
 # TODO add readcode for more languages
-readmetxt = wrap('Disk-based storage of a ragged array') + '\n' + \
-            wrap('====================================') + '\n\n' + \
-            wrap('This directory is a data store for a numeric ragged array. '
-                 'A ragged array (also called a jagged array) is a sequence '
-                 'of arrays that may vary in length in their first '\
-                 'dimension only. On disk, these arrays are concatenated '\
-                 'along their variable dimension. The easiest way to '\
-                 'access the data is to use the Darr library '\
-                 '(https://pypi.org/project/darr/) in Python, as '\
-                 'follows:') \
-            + '\n\n' \
-            + '>>> import darr\n' \
-            + ">>> a = darr.RaggedArray('path_to_array_dir')\n\n" + \
-            wrap("where 'path_to_array_dir' is the name of the array "
-              "directory, which is the one that also contains this README.")\
-               + "\n\n" + \
-            wrap('If Darr is not available, the data can also be read in '\
-                 'other environments, with a little more effort, using the '\
-                 'description or example code below.') + '\n\n\n' \
-            + 'Description of data storage\n' \
-            + '---------------------------\n' + \
-            wrap('There are two subdirectories, each containing an array '
-                 'stored in a self-explanatory format. See the READMEs in '
-                 'the corresponding directories to find out in detail out '
-                 'how to read them. Example code is provided below '
-                 'for a number of analysis environments, which in many cases '
-                 'is sufficient.') + '\n\n' + \
-            wrap('The subdirectory "values" holds the numerical data itself, '
-                 'where subarrays are simply appended along their variable '
-                 'length dimension (first axis). So the number of dimensions '
-                 'of the values array is one less than that of the ragged '
-                 'array. A particular subarray can be be retrieved using the '
-                 'appropriate start and end index along the first axis of the '
-                 'values array. These indices (counting from 0) are stored in '
-                 'a different 2-dimensional array in the subdirectory '
-                 '"indices". The first axis of the index array represents the '
-                 'sequence number of the subarray and the second axis '
-                 '(length 2) represents start and (non-inclusive) end '
-                 'indices to be used on the values array. To read the n-th '
-                 'subarray, read the nt-h start and end indices from the '
-                 'indices array and use these to read the array data from '
-                 'the values array.') + '\n\n\n'
+def readmetxt(ra):
+    n = len(ra)
+    ndsa  = len(ra.atom)
+    txt = wrap(f'This directory stores a numeric ragged array (also '
+               f'called a jagged array), which is a sequence of '
+               f'subarrays that may be multidimensional and that '
+               f'can vary in the length of their first dimension.') + ' \n\n'
+    txt += wrap(f'The ragged array can be read using the NumPy-based Python '
+                f'library Darr (https://pypi.org/project/darr/), which was '
+                f'used to create the data. If that is not available, you can '
+                f'use the code snippets below to read the data in a number of '
+                f'other environments. If code for your environment is not '
+                f'provided, use the description of how the data can be read '
+                f'in the next section.') + '\n\n'
+    txt += 'Description of ragged array\n' \
+           '===========================\n\n'
+    txt += wrap(f'This ragged array is a sequence of {n} '
+                f'subarrays, each of which is {ndsa + 1}-dimensional and '
+                f'can vary in the length of its first dimension. The array '
+                f'consists of {ra.dtype.name} numbers.') \
+                + ' \n\n'
+    dimtxt = f'The dimensions of the '
+    if len(ra) > 5:
+        dimtxt += f'first five '
+    if len(ra) > 6:
+        dimtxt += f'and last '
+    dimtxt = wrap(f'{dimtxt}subarrays is (subarray index: dimensions):') + '\n\n'
+    txt += dimtxt + dimensionstxt(ra, firstnmax=5) + '\n\n'
+    itxt = f'These index numbers are based on Python ' \
+           f'indexing, which starts at 0.'
+    if len(ra.atom) >0:
+        itxt = f'{itxt} Dimensions are based on row-major memory layout. When ' \
+               f'using the code provided below to read subarrays, dimensions ' \
+               f'will be inversed in column-major languages (see Note below).'
+    txt += wrap(itxt) + '\n\n'
+    txt += 'Description of storage on disk\n' \
+           '==============================\n\n'
+    txt += wrap('There are two subdirectories, "values" and "indices", each '
+                'containing an array stored in a self-explanatory format. '
+                'You first need to read these two arrays using '
+                'the information in the README.txt files in their '
+                'subdirectories. "values" holds the ragged array, where '
+                'subarrays are simply concatenated along their variable '
+                'length dimension (first axis). The n-th subarray can be '
+                'retrieved from the values array by using the appropriate '
+                'start and end index on the first axis of the values '
+                'array. These indices are stored in the two-dimensional '
+                'array in "indices". The first axis of the index array '
+                'corresponds to the sequence numbers of the subarrays, while '
+                'the length-2 second axis holds the start and end indices to '
+                'be used on the values array to retrieve a subarray. To '
+                'read the n-th subarray, read the nt-h start and end indices '
+                'from the indices array and use these to read the array data '
+                'from the values array. Note that the indices start counting '
+                'from zero, and end indices are non-inclusive.') + '\n\n\n'
+    return txt
+
+
+def dimensionstxt(ra, firstnmax=5):
+    end = min(len(ra), firstnmax)
+    lengths = np.diff(ra._indices[:end], axis=-1).flatten()
+    if len(ra.atom) > 0:
+        astr = str(ra.atom)[1:-1] + ')'
+    else:
+        astr = ')'
+    lines = []
+    for i, l in enumerate(lengths):
+        lines.append(f'    {i}: ({l}, {astr}')
+    if len(ra) > (firstnmax + 1):
+        lines.append('    ...')
+    if len(ra) > firstnmax:
+        lastdiff = np.diff(ra._indices[-1], axis=-1)[0]
+        lines.append(f'    {len(ra)-1}: ({lastdiff}, {astr}')
+    return '\n'.join(lines)
 
 
 def readcodetxt(ra):
@@ -365,20 +450,24 @@ def readcodetxt(ra):
 
     """
 
-    s = readmetxt
-    s += wrap(f'This ragged array has {len(ra)} subarrays. ') + '\n\n' + \
-         wrap(f'Example code for reading the data') + '\n' + \
+    s = readmetxt(ra)
+    s += wrap(f'Example code for reading the data') + '\n' + \
          wrap(f'=================================') + '\n\n'
     languages = (
+        ("Python with Darr:", "darr"),
         ("Python with Numpy (memmap):", "numpymemmap"),
         ("R:", "R"),
-        ("Matlab:", "matlab")
+        ("IDL/GDL", "idl"),
+        ("Julia (version >= 1.0):", "julia"),
+        ("Maple:", "maple"),
+        ("Matlab/Octave:", "matlab"),
+        ("Mathematica:", "mathematica")
     )
     for heading, language in languages:
         codetext = readcode(ra, language)
         if codetext is not None:
             s += f"{heading}\n{'-' * len(heading)}\n{codetext}\n"
-    return s
+    return f'{s}\n{shapeindexexplanationtextraggedarray}'
 
 
 def delete_raggedarray(ra):
