@@ -2,7 +2,8 @@
 languages to read the numeric information of a Darr array.
 
 """
-
+# Reading binary files:
+# ---------------------
 # mathematica: https://reference.wolfram.com/language/ref/BinaryRead.html
 # Octave: https://octave.org/doc/v4.2.0/Binary-I_002fO.html
 # Matlab: https://mathworks.com/help/matlab/ref/fread.html
@@ -10,13 +11,24 @@ languages to read the numeric information of a Darr array.
 # Julia: https://docs.julialang.org/en/release-0.4/manual/integers-and- floating-point-numbers/
 # Maple: https://www.maplesoft.com/support/help/maple/view.aspx?path=FileTools%2FBinary%2FRead
 
+# Row- vs column major:
+# ---------------------
+# IDL https://www.l3harrisgeospatial.com/docs/columns__rows__and_array.html
+# Matlab https://nl.mathworks.com/help/coder/ug/what-are-column-major-and-row-major-representation-1.html
+
+
+# Row-major: Mathematica, Numpy
+# Column-major: Julia, Matlab/Octave, R, Maple, IDL/GDL
 
 # Considerations
 # --------------
 # Matlab has memmapfile, but it will only read correctly if endianness of
-# data is the same as that of the host system.
+# data is the same as that of the host system, which we can't guarantee.
+# Same for Julia. Workaround code possible?
 
 import numpy as np
+from pathlib import Path
+from .utils import wrap
 
 def readcodedarr(numtype, shape, endianness, filepath='path_to_data_dir',
                   varname='a'):
@@ -69,11 +81,11 @@ typedescr_matlab = {'int8': 'int8',
                     'uint16': 'uint16',
                     'uint32': 'uint32',
                     'uint64': 'uint64',
-                    'float16': None,
+                    'float16': 'uint16',
                     'float32': 'float32',
                     'float64': 'float64',
-                    'complex64': None,
-                    'complex128': None,
+                    'complex64': '_special_case_',
+                    'complex128': '_special_case_',
                    }
 
 endianness_matlab = {'little': 'ieee-le',
@@ -84,6 +96,10 @@ def readcodematlab(numtype, shape, endianness,filepath='arrayvalues.bin',
     typedescr = typedescr_matlab[numtype]
     if typedescr is None:
         return None
+    elif numtype.startswith('complex'): # cannot be read directly by matlab
+        return readcodematlab_complex(numtype=numtype, shape=shape,
+                                      endianness=endianness, filepath=filepath,
+                                      varname=varname)
     endianness = endianness_matlab[endianness]
     shape = list(shape)[::-1]  # darr is always C order, Matlab is F order
     size = np.product(shape)
@@ -97,18 +113,62 @@ def readcodematlab(numtype, shape, endianness,filepath='arrayvalues.bin',
     else:  # ndim > 2, we need reshape to get multidimensional array
         ct += f"{varname} = reshape(fread(fileid, {size}, '*{typedescr}', " \
               f"'{endianness}'), {shape});\n"
+    if numtype == 'float16':  # cannot be read directly by matlab, via uint8
+        ct += f"{varname} = half.typecast({varname}); % may not work in " \
+              f"Octave yet\n"
     return ct + "fclose(fileid);\n"
 
+def readcodematlab_complex(numtype, shape, endianness,
+                           filepath='arrayvalues.bin',
+                           varname='a'):
+    if numtype=='complex128':
+        skip = 8
+        typedescr =  typedescr_matlab['float64']
+    elif numtype == 'complex64':
+        skip = 4
+        typedescr = typedescr_matlab['float32']
+    else:
+        raise ValueError(f"numtype '{numtype}' not a complex64 or complex128")
+    endianness = endianness_matlab[endianness]
+    shape = list(shape)[::-1]  # darr is always C order, Matlab is F order
+    size = np.product(shape)
+    ndim = len(shape)
+    ct = f"fileid = fopen('{filepath}');\n"
+    for subvarname, offset in zip(('re', 'im'),(0, skip)):
+        if offset > 0:
+            ct += f"fseek(fileid, {offset}, 'bof'); % to read imaginary " \
+                  f"numbers\n"
+        if ndim == 1:
+            ct += f"{subvarname} = fread(fileid, {size}, '*{typedescr}'," \
+                  f" {skip}," \
+                  f"'{endianness}');\n"
+        elif ndim == 2:
+            ct += f"{subvarname} = fread(fileid, {shape}, '*{typedescr}', " \
+                  f"{skip}, '{endianness}');\n"
+        else:  # ndim > 2, we need reshape to get multidimensional array
+            ct += f"{subvarname} = reshape(fread(fileid, {size}, " \
+                  f"'*{typedescr}', " \
+                  f"'{skip}, {endianness}'), {shape});\n"
+    ct += "fclose(fileid);\n"
+    return ct + f"{varname} = complex(re, im);\n"
 
+def readcodematlab_float16(shape, endianness,
+                           filepath='arrayvalues.bin',
+                           varname='a'):
+    ct = f"fileid = fopen('{filepath}');\n"
+
+
+# R does not support int64 but will read it OK for values that are valid for
+# int32: -2147483647 to 2147483647
 typedescr_r = {'int8': ('integer()', 1, 'TRUE'),
                'int16': ('integer()', 2, 'TRUE'),
                'int32': ('integer()', 4, 'TRUE'),
                'int64': ('integer()', 8, 'TRUE'),
                'uint8': ('integer()', 1, 'FALSE'),
                'uint16': ('integer()', 2, 'FALSE'),
-               'uint32': ('integer()', 4, 'FALSE'),
-               'uint64': ('integer()', 8, 'FALSE'),
-               'float16': ('numeric()', 2, 'TRUE'),
+               'uint32': None,
+               'uint64': None,
+               'float16': None,
                'float32': ('numeric()', 4, 'TRUE'),
                'float64': ('numeric()', 8, 'TRUE'),
                'complex64': None,
@@ -119,7 +179,9 @@ endianness_r = {'little': 'little',
                 'big': 'big'}
 
 def readcoder(numtype, shape, endianness, filepath='arrayvalues.bin',
-              varname='a'):
+              varname='a', ignoreint64=False):
+    if numtype == 'int64' and not ignoreint64:
+        return None
     typedescr = typedescr_r[numtype]
     if typedescr is None:
         return None
@@ -127,11 +189,11 @@ def readcoder(numtype, shape, endianness, filepath='arrayvalues.bin',
     what, size, signed = typedescr
     shape = shape[::-1]  # darr is always C order, R is F order
     n = np.product(shape)
-    ct = f'fileid = file("{filepath}", "rb")\n' \
-         f'{varname} = readBin(con=fileid, what={what}, n={n}, size={size}, ' \
+    ct = f'fileid <- file("{filepath}", "rb")\n' \
+         f'{varname} <- readBin(con=fileid, what={what}, n={n}, size={size}, ' \
          f'signed={signed}, endian="{endianness}")\n'
     if len(shape) > 1:
-        ct += f'{varname} = array(data={varname}, dim=c{shape}, ' \
+        ct += f'{varname} <- array(data={varname}, dim=c{shape}, ' \
             f'dimnames=NULL)\n'
     return ct + 'close(fileid)\n'
 
@@ -265,12 +327,66 @@ def readcodemaple(numtype, shape, endianness, filepath='arrayvalues.bin',
         return None
     ct = f'{varname} := FileTools[Binary][Read]("{filepath}", {typedescr}, ' \
          f'byteorder={endianness}, output=Array);\n'
+    ct += f'FileTools[Binary][Close]("{filepath}");\n'
     ndim = len(shape)
     if ndim > 1:
         shape = list(shape[::-1])
         ct += f'{varname} := ArrayTools[Reshape]({varname}, {shape});\n'
     return ct
 
+typedescr_python = {'int8': 'b',
+                   'int16': 'h',
+                   'int32': 'l',
+                   'int64': 'q',
+                   'uint8': 'B',
+                   'uint16': 'H',
+                   'uint32': 'L',
+                   'uint64': 'Q',
+                   'float16': None,
+                   'float32': 'f',
+                   'float64': 'd',
+                   'complex64': 'f', # special case, we create two f arrays
+                   'complex128': 'd', # special case, we creats two d arrays
+                  }
+
+endianness_python = {'little': '<',
+                    'big': '>'}
+
+def readcodepython(numtype, shape, endianness, filepath='arrayvalues.bin',
+                  varname='a'):
+    """We only support 1D arrays. Complex types are supported but lead to
+    separate real and imaginary arrays in Python, as the array type does not
+    support complex numbers natively"""
+
+    typeletter = typedescr_python[numtype]
+    endianness = endianness_python[endianness]
+    if typeletter is None:
+        return None
+    if len(shape) > 1:
+        return None
+    size = shape[0]
+    if numtype.startswith('complex'):
+        size *= 2 # we split real and imaginary numbers
+    typedescr = f"{endianness}{size}{typeletter}"
+    ct = f"import array\nimport struct\n"
+    if numtype.startswith('complex'):
+        fptype = {'f': 'float', 'd': 'double' }[typeletter]
+        ct += f"# file holds complex values but we need to read them as" \
+              f" {fptype} type\n"
+    ct += f"with open('arrayvalues.bin', 'rb') as f:\n" \
+          f"    {varname} = array.array('{typeletter}', " \
+          f"struct.unpack('{typedescr}', f.read()))\n"
+    if numtype.startswith('complex'):
+        ct += f"# array '{varname}' has real and imaginary values at " \
+              f"alternating positions\n" \
+              f"# we can split them into separate arrays\n" \
+              f"real = array.array('{typeletter}', ({varname}[i] for i in " \
+              f"range(0, len({varname}), 2)))\n"\
+              f"imag = array.array('{typeletter}', ({varname}[i] for i in " \
+              f"range(1, len({varname}), 2)))\n"
+    return ct
+
+
 
 readcodefunc = {
         'darr': readcodedarr,
@@ -282,11 +398,12 @@ readcodefunc = {
         'maple': readcodemaple,
         'numpy': readcodenumpy,
         'numpymemmap': readcodenumpymemmap,
+        'python': readcodepython,
         'R': readcoder,
 }
 
 
-def readcode(da, language, filepath='arrayvalues.bin', varname='a'):
+def readcode(da, language, abspath=False, basepath=None, varname='a',**kwargs):
     """Produces the code to read the Darr array `da` in a given programming
     language.
 
@@ -295,7 +412,12 @@ def readcode(da, language, filepath='arrayvalues.bin', varname='a'):
     da: Darr array
     language: str
         A supported language, such as 'julia', 'R', or 'matlab'
-
+    abspath: bool
+        Should the paths to the data files be absolute or not? Default:
+        True.
+    basepath: str or pathlib.Path or None
+        Path relative to which the binary array data file should be
+        provided. Default: None.
     Returns
     -------
     A string with code
@@ -307,10 +429,32 @@ def readcode(da, language, filepath='arrayvalues.bin', varname='a'):
     numtype = d['numtype']
     shape = d['shape']
     endianness = d['byteorder']
+    if abspath:
+        filepath = da._datapath.absolute().resolve()
+    elif basepath is not None:
+        filepath = Path(basepath) / da._datapath.name
+    else:
+        filepath = Path(da._datapath.name)
+    filepath = filepath.as_posix()
     return readcodefunc[language](numtype=numtype, shape=shape,
                                   endianness=endianness, filepath=filepath,
-                                  varname=varname)
+                                  varname=varname, **kwargs)
 
 
 def promptify_codetxt(codetxt, prompt=">>> "):
     return "\n".join([f"{prompt}{l}" for l in codetxt.splitlines()]) + '\n'
+
+
+shapeexplanationtextarray = \
+     f'Notes on dimensions and indexing of arrays\n' \
+     f'==========================================\n\n' + \
+wrap(f'The dimensions stated in the format description above are based on a '
+     f'row-major memory layout where the *last* dimension is the one that '
+     f'varies most rapidly with memory address. However, in some languages '
+     f'arrays are based on column-major memory layout. To keep things '
+     f'efficient, the code examples above do not change the memory layout '\
+     f'when reading the array in a different language. This means that in '
+     'column-major languages, the dimension axes will be *inversed*. Row-major '
+     f'languages are: Python and Mathematica. Columns-major languages are: '
+     f'Julia, Matlab/Octave, R, Maple, and IDL/GDL. \n')
+
