@@ -1,6 +1,8 @@
 import unittest
 import json
 import numpy as np
+import shutil
+from pathlib import Path
 from darr.utils import fit_frames, write_jsonfile, product
 from darr.utils import tempdir, tempdirfile
 
@@ -123,5 +125,56 @@ class FitChunks(unittest.TestCase):
         self.assertRaises(ValueError, fit_frames, totallen=3, chunklen=2,
                           steplen=-1.)
 
+class CreateTempDir(unittest.TestCase):
+
+    def test_ispath(self):
+        with tempdir() as td:
+            self.assertIsInstance(td, Path)
+            self.assertTrue(td.exists())
+            self.assertTrue(td.is_dir())
+
+    def test_isdeleted(self):
+        with tempdir() as td:
+            pass
+        self.assertFalse(td.exists())
+
+    def test_report(self):
+        with tempdir(report=True) as td:
+            pass # see at least if this doesn't fail
+
+    def test_keep(self):
+        with tempdir(keep=True) as td:
+            pass
+        self.assertTrue(td.exists())
+        shutil.rmtree(td)
+
+
+
+class CreateTempDirFile(unittest.TestCase):
+
+    def test_ispath(self):
+        with tempdirfile() as tdf:
+            td = tdf.parent
+            self.assertIsInstance(tdf, Path)
+            self.assertTrue(td.is_dir())
+            self.assertFalse(tdf.exists())
+
+    def test_isdeleted(self):
+        with tempdirfile() as td:
+            pass
+        self.assertFalse(td.exists())
+        self.assertFalse(td.parent.exists())
+
+    def test_report(self):
+        with tempdirfile(report=True) as td:
+            pass # see at least if this doesn't fail
+
+    def test_keep(self):
+        with tempdirfile(keep=True) as td:
+            pass
+        self.assertTrue(td.parent.exists())
+        shutil.rmtree(td.parent)
+
+
 if __name__ == '__main__':
     unittest.main()
\ No newline at end of file
