@@ -122,6 +122,8 @@ def wrap(s):
 
 
 
+#TODO avoid double code in next two functions; and do we really need both
+# after switching to shutil for removinf dirtree?
 @contextmanager
 def tempdir(dirname='.', keep=False, report=False):
     """Yields a temporary directory which is removed when context is closed."""
@@ -135,46 +137,18 @@ def tempdir(dirname='.', keep=False, report=False):
     finally:
         if not keep:
             shutil.rmtree(tempdirname)
-            if report:
-                print('removed temp dir {}'.format(tempdirname))
-
-# FIXME does not yield a name but a path
-# @contextmanager
-# def tempdirfile(dirname=None, keep=False, report=False):
-#     """Yields a file named "tempfile" in a temporary directory which is
-#     removed when context is closed."""
-#     tempdirname = None
-#     tempfilename = None
-#     try:
-#         tempdirname = tf.mkdtemp(dir=dirname)
-#         if report:
-#             print(f'created temporary directory {tempdirname}')
-#         tempfilename = Path(tempdirname) / "tempfile"
-#         yield tempfilename
-#     except:
-#         raise
-#     finally:
-#         if not keep:
-#             if tempfilename is not None:
-#                 for root, dirs, files in os.walk(tempfilename):
-#                     for file in files:
-#                         os.remove(Path(root) / file)
-#                 for root, dirs, files in os.walk(tempfilename):
-#                     for dir in dirs:
-#                         os.rmdir(Path(root) / dir)
-#                 try:
-#                     os.rmdir(tempfilename)
-#                 except FileNotFoundError:
-#                      pass
-#             if tempdirname is not None:
-#                 os.rmdir(tempdirname)
-#             if report:
-#                 print('removed tempdir {}'.format(tempdirname))
+        if report:
+            if keep:
+                verb = 'kept'
+            else:
+                verb = 'removed'
+            print(f'{verb} temporary directory {tempdirname}')
 
 @contextmanager
 def tempdirfile(dirname=None, keep=False, report=False):
-    """Yields a file named "tempfile" in a temporary directory which is
-    removed when context is closed."""
+    """Yields a filename "tempfile" in a temporary directory which is
+    removed when context is closed. Note that the directory is created,
+    but the file "tempfile" not."""
     tempdirname = None
     try:
         tempdirname = tf.mkdtemp(dir=dirname)
@@ -192,4 +166,4 @@ def tempdirfile(dirname=None, keep=False, report=False):
                 verb = 'kept'
             else:
                 verb = 'removed'
-            print(f'{verb} tempdir {tempdirname}')
+            print(f'{verb} temporary directory {tempdirname}')
